/**
 * Contains:   Editor login plug-in for blojsom.
 * Written by: John Anderson (for addtl writers check CVS comments).
 * Copyright:  © 2004 Apple Computer, Inc., all rights reserved.
 * Note:       When editing this file set PB to "Editor uses tabs/width=4".
 *
 * $Id: AppleBlojsomVersion.java,v 1.22 2004/10/20 23:32:22 johnan Exp $
 */ 
package com.apple.blojsom.util;

/**
 * AppleBlojsomVersion
 * 
 * @author John Anderson
 * @version $Id: AppleBlojsomVersion.java,v 1.22 2004/10/20 23:32:22 johnan Exp $
 */
public class AppleBlojsomVersion {
	
	// Constants
    public static final String appleBlojsomRevision = "blojsom-23";
    
}
