img {
    border: 0px;
}

body
{
	color: #333333;
	background-color: white;
	margin-top: 24pt;
	margin-left: 50pt;
	margin-right: 50pt;
	margin-bottom: 36pt;
	font-family: Georgia, "Times New Roman", Times, serif;
}


h1
{
	font-family: Helvetica, Arial, sans-serif;
	color: #000000;
	margin-top: 0pt;
	margin-bottom: 2pt;
	padding-top: 0pt;
}

h3
{
	font-family: Helvetica, Arial, sans-serif;
	color: #000000;
	margin-top: 0pt;
	margin-bottom: 4pt;
	padding-top: 0pt;
}


a
{
	color: #006600;
	text-decoration: none;
}

a:hover
{
	color: #009900;
	text-decoration: underline;
}

blockquote {
    border: 2px dotted;
    background-color: #FFFFFF; 
    padding: 15px;
    margin: 10px 10px;
    color: #333;
}

.navigation
{
	font-family: Helvetica, Arial, sans-serif;
	font-size: smaller;
	margin-bottom: 0pt;
	padding-bottom: 0pt;
}

.copyright
{
	color: #AAAAAA;
	font-size: smaller;
	margin-top: 36pt;
}

.weblognav
{
	float: right;
	font-family: Helvetica, Arial, sans-serif;
	font-size: smaller;
}

.weblogtitle
{
	font-family: Helvetica, Arial, sans-serif;
	font-weight: bold;
	font-size: larger;
	margin-bottom: 0pt;
	padding-bottom: 0pt;
}

.weblogtitle2
{
	font-family: Helvetica, Arial, sans-serif;
	font-weight: bold;
	font-size: larger;
	margin-bottom: 0pt;
	margin-top: 1pt;
	padding-bottom: 0pt;
}

.weblogdateline
{
	font-family: Helvetica, Arial, sans-serif;
	font-weight: normal;
	font-size: smaller;
	color: #555555;
	margin-top: 0pt;
	padding-top: 0pt;
}

.refererhistory
{
	font-family: Helvetica, Arial, sans-serif;
	font-weight: normal;
	font-size: 75%;
	color: #555555;
	margin-top: 0pt;
	margin-top: 0pt;
	padding-top: 0pt;
}

.weblogpostedbyline
{
	font-family: Helvetica, Arial, sans-serif;
	font-size: smaller;
	color: #000000;
	margin-top: 0pt;
	padding-top: 0pt;
}

.weblogbottomline
{
	font-family: Helvetica, Arial, sans-serif;
	font-size: smaller;
	margin-top: 0pt;
	padding-top: 0pt;
	margin-bottom: 20px;
	text-align: right;
}

.entrystyle
{
	margin-bottom: 20px;
	text-align: left;
	background-color: #EEEEEE;
}

.commentstyle
{
	margin-bottom: 20px;
	text-align: left;
	background-color: #EEEEEE;
}

.smalltext
{
	font-size: 50%;
	font-weight: bold;
}


.searchtext
{
    margin-top: 0px;
	margin-bottom: 0px;
	align: right;
}

.searchimage
{
    margin-top: 0px;
	margin-bottom: 0px;
	vertical-align: middle;
}

#searchbox{
	float:right;
	width : 200px;
	background-color: #FFFFFF;
	border-left:0px solid #000000;
	border-right:0px solid #000000;
	border-bottom:0px solid #000000;
	border-top:0px solid #000000;
	padding:0px;
	margin-top:5px;
	margin-bottom:0px;
	margin-right:0px;
	align: right;
}

.searchform
{
    margin-top: 2px;
	margin-bottom: 2px;
}


.calendarcaption {
   font-family: Helvetica, Arial, sans-serif;
   text-align:center;
   font-weight: bold;
   font-size: larger;
   margin-bottom: 0pt;
   margin-top: 1pt;
   padding-bottom: 0pt;
}

.calendarbox {
   background-color:#FFFFFF;
   border-left:1px solid #000000;
   border-right:1px solid #000000;
   border-bottom:1px solid #000000;
   border-top:1px solid #000000;
   padding:5px;
   margin-bottom: 15px;
}

.calendarcolumn {
    font-family: Helvetica, Arial, sans-serif;
	font-size:  10px;
	text-align: center;
}

.tablemaxwidth {
   width: 100%;
   vertical-align: top;
}

.tablemaxpxwidth {
   padding:5px;
}

.searchhighlight {
   background-color:#FFFF00;
   color: #000000;
}