feed {
  display:block;
  font-family:verdana, sans-serif;
  margin:2%;
}

info {
  margin:20px;
  background:yellow;
  text-align:center;
  display:block;
  padding:10px;
  font-size:85%;
}

title {
  font-size:150%;
  color:#000000;
  display:block;
  font-weight:bold;
}

tagline {
  display:block;
  font-size:90%;
}

link {
  font-size:90%;
}

id {
  display:none;
}

modified {
  display:none;
}

generator {
  font-size:90%;
  display:none;
  margin:0px 0px 30px 0px;
}

entry title {
  font-size:125%;
  font-weight:bold;
  color:#003366;
  display:block;
  padding:15px 1% 0px 3%;
  margin:10px 0px 0px 0px;
  border-top:solid 1px #dddddd;
  background:#eeeeee;
}

entry modified {
  display:inline;
}

created {
  display:none;
}

issued {
  display:none;
}

content {
  background:#eeeeee;
  display:block;
  padding:5px 3% 10px 3%;
  border-bottom:solid 1px #999999;
}



entry {
  border-top:dotted 1px #999999;
  margin:30px 0px 5px 0px;
  padding:5px;
  display:block;
  background:#ffffff;
  padding:5px 0px 10px 0px;
  margin:10px 10px 35px 10px;
  
}

entry link {
  color:#aaaaaa;
  font-size:80%;
}

issued,modified,created,name,id {
  color:#999999;
  font-size:80%;
  margin-top:25px;
}
